// --- [ BMP085_main.c ] -------------------------------------------------------
//
//       tab = 3 - 15.05.11


#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <math.h>					// To calculate altitude

#define EXTERN

#include "bmp085/rs232.h"
#include "bmp085/TWI_Master_c.h"
#include "global.h"
#include "bmp085/bmp085_a.h"


#if USE_SHT
#include "sht/libsht.h"
#endif

// --- Prototypen --------------------------------------------------------------

void make_dez_point(void);
void make_dez_point2(void);

// --- Globale Variablen -------------------------------------------------------

// ----------------------------------------------------------------------------
int main(void)
{	
	long temperature = 0;
	long pressure = 0;
/*	long pressure_NN;
	double altitude;
	double temp = 0;
*/	
	
	rs232_init();
	puts_p(PSTR("\n\rStart\n\r"));
//	put_c(13);
	
	TWI_Master_Initialise();
	asm volatile("sei");					// fr TWI erforderlich
	
	twi_buffer[0] = BMP085_TWI_ADR;
	TWI_MA_Start_Transceiver_With_Data(twi_buffer, 1);

// --- Beim Start die gewhlten Einstellungen zur Kontrolle anzeigen ---
#ifdef DEBUG	
	uint16_t ii;
	put_c(13);
	puts_p(PSTR("TWI_Adr: 0x"));		// Meldung, dass TWI-Adresse gefunden
	put_s(utoa(BMP085_TWI_ADR, cv, 16));
	
	if (TWI_LastTransOK())
		{
		puts_p(PSTR(" found\n\r"));
		}
	else
		{
		puts_p(PSTR(" missing - quit\n\r"));
		while(1);
		}
	
	put_c(13);

	ii = BMP085_Read_Data(BMP085_CHIP_ID_REG);  // Chip-ID ausgeben
	put_s("Chip_ID: 0x");
	put_s(itoa(ii >> 8, cv, 16));
	put_c(13);
	
	ii = BMP085_Read_Data(BMP085_VERSION_REG);  // Chip-Version ausgeben
	put_s("\n\rVersion: 0x");
	put_s(itoa((ii >> 8), cv, 16));
	put_c(13);
	
	puts_p(PSTR("\n\rOSS: "));								// OSS ausgeben
	put_s(utoa(BMP085_OSS, cv, 10));
	puts_p(PSTR("\n\r - reading uPressure: "));
#if READ_BIT == 19
	puts_p(PSTR("19"));								// wieviele Bits werden gelesen
#elif  READ_BIT == 16
	puts_p(PSTR("16"));
#else
	#error READ_BIT in bmp085_a.h nicht gesetzt
#endif
	puts_p(PSTR(" Bit\n\r"));
	put_c(13);
	
	puts_p(PSTR("Software Samples: "));			// wieviele Software Samples
	put_s(utoa(SOFTWARE_SAMPLES, cv, 10));	
	put_c(13);
	
	puts_p(PSTR("\n\rFloating P_Average: "));		// gibts Floating Average ?
#if FLOATING_AVERAGE == TRUE
	puts_p(PSTR("ON -> \n\r"));
	put_s(utoa(AVERAGE_SAMPLES, cv, 10));
#else
	puts_p(PSTR("OFF\n\r"));
#endif
	put_c(13);
	put_c(13);
#endif
	
	delay_10ms(5);									// Wartezeit nach power-up 
														// bis zur ersten Kontaktaufnahme
	BMP085_Read_Calibration();
	
#if USE_SHT
	int dewpoint;
	unsigned char sht_error=0;
	sht_error=sht_softreset(SHT_DATA_PIN);  // SHT wird dabei auch auf 8/12bit Mode gesetzt
//	if (sht_error) puts_p(PSTR("SHT Error (SoftReset)\r\n"));
#endif

	while(1)
		{
		BMP085_Convert(&temperature, &pressure);
	
		puts_p(PSTR(" T: "));
		ltoa(temperature, cv, 10);
		make_dez_point();
		put_s(cv);
		
		puts_p(PSTR(" C - P: "));
		ltoa(pressure, cv, 10);
		make_dez_point2();
		put_s(cv);
		puts_p(PSTR(" hPa "));
		
		// convert Pressur to altitude
/*		temp = (double) pressure/101325;
		temp = 1-pow(temp, 0.19029);
		altitude = round(44330*temp);
		//printf("Altitude: %ldm\n\n", altitude);
*/		
/*		// convert Pressure to sealevel
		altitude = 90.0;					// Angenommene Hhe
		temp = 1.0 - (altitude / 44330);
		temp =  pow(temp, 5.255);
		pressure_NN = pressure / temp;
		puts_p(PSTR(" - NN: "));
		ltoa(pressure_NN, cv, 10);
		make_dez_point2();
		put_s(cv);
		puts_p(PSTR(" hPa"));
		put_c(13);
		put_c(13);
*/

#if USE_SHT
// ******************************************************************
// SHT-Sensor (SHT 11) abfragen
// ******************************************************************
		unsigned char sht_checksum, sht_error=0;
		sht_value humidity;
		sht_value temperature;

		// liest humi und temp vom SHT Sensor und berechnet dewpoint
		sht_error=sht_measure(&humidity, &sht_checksum, HUMI, SHT_DATA_PIN);
//		if (sht_error) puts_p(PSTR("SHT Error (HUMI)\r\n"));

		sht_error=sht_measure(&temperature, &sht_checksum, TEMP, SHT_DATA_PIN);
//		if (sht_error) puts_p(PSTR("SHT Error (TEMP)\r\n"));

   		sht_raw_to_physical(&humidity, &temperature);
		dewpoint = calc_dewpoint(humidity.p/10,temperature.p);

		puts_p(PSTR(" SHT-T: "));
		ltoa(temperature.p, cv, 10);
		make_dez_point();
		put_s(cv);
		
		puts_p(PSTR(" C - Humi: "));
		ltoa(humidity.p/10, cv, 10);
		put_s(cv);
		puts_p(PSTR(" %"));

		puts_p(PSTR(" Dew: "));
		ltoa(dewpoint, cv, 10);
		make_dez_point();
		put_s(cv);
		puts_p(PSTR(" C \n\r"));
		
	#endif		// USE_SHT


		delay_10ms(250);
	}
}

// -----------------------------------------------------------------------------
// Verzgerung von 10ms bis 2.55 Sekunden
void delay_10ms(uint8_t t)
{
while(t--) _delay_ms(10);
}

// ----------------------------------------------------------------------------
// setzt einen Dezimalpunkt vor dem letzten Zeichen
void make_dez_point(void)
{
uint8_t i = 0;

while (cv[i]) i++;
cv[i + 1 ] = 0;
cv[i] = cv[i - 1];
cv[i - 1] = DEZIMAL_TRENNZEICHEN;
}

// ----------------------------------------------------------------------------
// setzt einen Dezimalpunkt vor dem vorletzten Zeichen
void make_dez_point2(void)
{
uint8_t i = 0;

while (cv[i]) i++;

if (i < 2) return;

cv[i + 1 ] = 0;
cv[i] = cv[i - 1];
cv[i-1] = cv[i - 2];
cv[i - 2] = DEZIMAL_TRENNZEICHEN;
}


// --- [ eof ] -----------------------------------------------------------------
