
/***********************************************************************************
Sensirion SHTxx Sensor Library 0v2.2

***********************************************************************************/
/* Copyright Notice
 	This library for the SHT temperature and humidity sensors is based on the 
	application datasheet "Sample Code humidity sensor SHTxx" from Sensirion. 
	(c) Timo Dittmar
		(For now. If I get enough feedback and a positive answer from
		Sensirion I plan to release the code under GPL)
	Use without any warranty
*/


/* History
	Date, Version, Comment 
	2006-07-14	NA		- Initial conversion from Sensirion application note
						- inserted test code
	2006-07-17	NA		- Adjusted timing in measurement
						- changed data transfer in calc_sht; 
	2006-07-20	0v1rc	- Included heating element code 
	2006-08-05	0v1		- Some additional cleanup
						- homogenized function names;
	2006-08-13	0v2rc	- included functions for measuring with recurring interrupt
						- fewer orthographic errors :-)
						- first implementation of crc check
	2006-08-27	0v2		- second release
	2009-02-05	0V2.2	- include delay_ms function (cni) deleted ***GWR***
	2009-06-18	0V2.2	- change  float- to integer computation ***GWR***
	2009-06-18	0V2.2	- 8/12 bit data from SHT supported only ***GWR***
*/


#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <util/delay.h>

#include "..\global.h"
#include "libsht.h"

//----------------------------------------------------------------------------------
unsigned char sht_write_byte(unsigned char sht_value, unsigned char pin)
//----------------------------------------------------------------------------------
// writes a byte on the Sensibus and checks the acknowledge
{
	unsigned char i;
	unsigned char error=0;
	
	MAKE_SHT_DATA_PIN_OUTPUT(pin);
	asm volatile ("nop"::);
	
	for (i=0x80;i>0;i/=2) 				//shift bit for masking
	{
		if (i & sht_value) 
		{	
			SET_SHT_DATA(pin);			//masking value with i , write to SENSI-BUS
		}
	 	else 
		{
			CLEAR_SHT_DATA(pin);
		}
		SET_SHT_SCK; //clk for SENSI-BUS
		asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
		CLEAR_SHT_SCK;
		asm volatile ("nop"::);
		asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	}
	
	SET_SHT_DATA(pin); //release DATA-line
	MAKE_SHT_DATA_PIN_INPUT(pin);
	asm volatile ("nop"::);
		asm volatile ("nop"::);
		asm volatile ("nop"::);
		asm volatile ("nop"::);
	SET_SHT_SCK; 					//clk #9 for ACK
	asm volatile ("nop"::);
	asm volatile ("nop"::);
		asm volatile ("nop"::);
		asm volatile ("nop"::);
		asm volatile ("nop"::);

	if (SHT_DATA(pin)) 
	{
		error =1; //check ack (DATA will be pulled down by SHT11)
	}
	CLEAR_SHT_SCK; // move to pre-if
	return error; //error=1 in case of no acknowledge
}


//----------------------------------------------------------------------------------
unsigned char sht_read_byte(unsigned char ack, unsigned char pin)
//----------------------------------------------------------------------------------
// reads a byte form the Sensibus and gives an acknowledge in case of "ack=1"
{
	unsigned char i;
	unsigned char val=0;
	MAKE_SHT_DATA_PIN_OUTPUT(pin);
	asm volatile ("nop"::);
		asm volatile ("nop"::);
		asm volatile ("nop"::);
		asm volatile ("nop"::);
	SET_SHT_DATA(pin); //release DATA-line
	MAKE_SHT_DATA_PIN_INPUT(pin);
	asm volatile ("nop"::);
		asm volatile ("nop"::);
		asm volatile ("nop"::);
		asm volatile ("nop"::);
	
	for (i=0x80;i>0;i/=2) //shift bit for masking
	{
		SET_SHT_SCK; //clk for SENSI-BUS
		asm volatile ("nop"::);
		asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
		if (SHT_DATA(pin)) val=(val | i); //read bit
		CLEAR_SHT_SCK;
		asm volatile ("nop"::);
		asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	}
	
	MAKE_SHT_DATA_PIN_OUTPUT(pin);
	asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	if (ack) CLEAR_SHT_DATA(pin); 
	else SET_SHT_DATA(pin); // Sent ack
	
	SET_SHT_SCK; //clk #9 for ack
	asm volatile ("nop"::);
	asm volatile ("nop"::);
	asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	
	CLEAR_SHT_SCK;
	asm volatile ("nop"::);
	asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	SET_SHT_DATA(pin); //release DATA-line
	return val;
}



//----------------------------------------------------------------------------------
void sht_transstart(unsigned char pin)
//----------------------------------------------------------------------------------
// generates a transmission start
{
	MAKE_SHT_DATA_PIN_OUTPUT(pin);
	asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	SET_SHT_DATA(pin); CLEAR_SHT_SCK; //Initial state
	asm volatile ("nop"::);
	asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	SET_SHT_SCK;
	asm volatile ("nop"::);
	asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	CLEAR_SHT_DATA(pin);
	asm volatile ("nop"::);
	asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	CLEAR_SHT_SCK;
	asm volatile ("nop"::);
	asm volatile ("nop"::);
	asm volatile ("nop"::);
	asm volatile ("nop"::);
	asm volatile ("nop"::);
	asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	SET_SHT_SCK;
	asm volatile ("nop"::);
	asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	SET_SHT_DATA(pin);
	asm volatile ("nop"::);
	asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	CLEAR_SHT_SCK;
}


//----------------------------------------------------------------------------------
void sht_connectionreset(unsigned char pin)
//----------------------------------------------------------------------------------
// communication reset: DATA-line=1 and at least 9 SCK cycles followed by transstart
{
	unsigned char i;
	MAKE_SHT_DATA_PIN_OUTPUT(pin);
	asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	
	SET_SHT_DATA(pin); CLEAR_SHT_SCK; //Initial state
	
	for(i=0;i<9;i++) //9 SCK cycles
	{
		SET_SHT_SCK;
		asm volatile ("nop"::);
		asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
		
		CLEAR_SHT_SCK;
		asm volatile ("nop"::);
		asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
	}
	sht_transstart(pin); //transmission start
}



//----------------------------------------------------------------------------------
unsigned char sht_softreset(unsigned char pin)
//----------------------------------------------------------------------------------
// resets the sensor by a softreset
{
	// rockclimber
	MAKE_SHT_SCK_PIN_OUTPUT;
	asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);
			asm volatile ("nop"::);

	unsigned char error=0;
//	unsigned char checksum;
	unsigned char status;
	sht_connectionreset(pin); //reset communication
	error+=sht_write_byte(RESET_SHT, pin); //send RESET_SHT-command to sensor
//	sht_read_statusreg(&status, &checksum, pin);
	status |= (LOWRES_BIT);				// erst mal kompatibel mit s555HT sensor
	sht_write_statusreg(&status, pin);	// erst mal kompatibel mit s555HT sensor
	return error; //error=1 in case of no response form the sensor
}



//----------------------------------------------------------------------------------
unsigned char sht_read_statusreg(unsigned char *p_sht_value, unsigned char *p_checksum, unsigned char pin)
//----------------------------------------------------------------------------------
// reads the status register with checksum (8-bit)
{
	unsigned char error=0;
	
	sht_transstart(pin); //transmission start
	error=sht_write_byte(STATUS_REG_R, pin); //send command to sensor
	*p_sht_value=sht_read_byte(ACK, pin); //read status register (8-bit)
	*p_checksum=sht_read_byte(noACK, pin); //read checksum (8-bit)
	return error; //error=1 in case of no response form the sensor
}



//----------------------------------------------------------------------------------
unsigned char sht_write_statusreg(unsigned char *p_sht_value, unsigned char pin)
//----------------------------------------------------------------------------------
// writes the status register with checksum (8-bit)
{
	unsigned char error=0;
	sht_transstart(pin); //transmission start
	error+=sht_write_byte(STATUS_REG_W, pin);//send command to sensor
	error+=sht_write_byte(*p_sht_value, pin); //send value of status register
	return error; //error>=1 in case of no response form the sensor
}



//----------------------------------------------------------------------------------
unsigned char sht_measure(sht_value *p_sht_value, unsigned char *p_checksum, unsigned char mode, unsigned char pin)
//----------------------------------------------------------------------------------
// makes a measurement (humidity/temperature) with checksum
{
	unsigned char error=0;
	unsigned char i;

	sht_value this_sht_value;
	/* sht_transstart(pin); //transmission start */
	sht_connectionreset(pin);
	switch(mode)
	{ //send command to sensor
		case TEMP : error+=sht_write_byte(MEASURE_TEMP, pin); /*usart_write("TEMP: ");*/ break;
		case HUMI : error+=sht_write_byte(MEASURE_HUMI, pin); /*usart_write("HUMI: ");*/ break;
		default : break;
	}

	for (i=0;i<110;i++) {
		_delay_ms(5);  				 // the 110ms for 12bit measuremnt are only a rough estimate
		if(SHT_DATA(pin)==0) break; //wait until sensor has finished the measurement
	}

	if(SHT_DATA(pin)) error+=1; // or timeout is reached 

	this_sht_value.i = 256*sht_read_byte(ACK, pin); //read the first byte (MSB)
	this_sht_value.i +=sht_read_byte(ACK, pin); //read the second byte (LSB)
		
	*p_checksum =sht_read_byte(noACK, pin); //read checksum
	*(p_sht_value)= this_sht_value;
	return error;
}




//----------------------------------------------------------------------------------------
void sht_raw_to_physical(sht_value *p_humidity ,sht_value *p_temperature)
//----------------------------------------------------------------------------------------
//	2009-06-18	0V2.2	- change  float- to integer computation ***GWR***
// calculates temperature [C] and humidity [%RH]
// input : 	humi [Ticks] (8 bit)
// 			temp [Ticks] (12 bit)
// output:	humi [ %RH times 10 ]
// 			temp [in C times 10]
{ 	
	unsigned int rh_lin;	// rh_lin: Humidity linear
	int rh_calc;	// rh_calc: used for calculation purposes
	int rh_true;	// rh_true: Temperature compensated humidity
	int t_C; // t_C : Temperature in Zehntel Grad[C]
	t_C = (4*(*p_temperature).i)/10 - 401;		//calc. Temperature from ticks to [C]

	if ( ((*p_humidity).i) <= 107 ) {			
	rh_lin = (143*(*p_humidity).i -512)/256;	
	}
	else {
	rh_lin = (111*(*p_humidity).i + 2893)/256; 
	}
//	usart_write("T %i F_roh %i F_lin %i \r\n",t_C,(*p_humidity).i, rh_lin);

//	rh_true=(t_C-25)*(0.01+0.00128*(*p_humidity).i)+rh_lin; //calc. Temperature compensated humidity [%RH]
	rh_calc = (t_C/10-25);
	rh_true = (((*p_humidity).i)*128/100 + 10);
//	usart_write("rh_calc %i5 rh_true %i korr %i \r\n",rh_calc,rh_true,(rh_calc*rh_true)/1000);

	rh_true = (rh_true * rh_calc)/100 + rh_lin*10;
	if(rh_true >= 1000)rh_true = 1000;		//cut if the value is outside of
	if(rh_true <= 1)rh_true = 1; 		//the physical possible range

	(*p_temperature).p=t_C;  //return temperature [C]
	(*p_humidity).p=rh_true; //return humidity[%RH]
}


//--------------------------------------------------------------------
//int calc_dewpoint(int h,int t); this is from tuxgraphics.org
//--------------------------------------------------------------------
// calculates dew point
// input: humidity [%RH], temperature [C] times 10
// output: dew point [C] times 10
//--------------------------------------------------------------------
// first, we need log10 function:
// this is an approximation of 100*log10(x) and does not need the math
// library. The error is less than 5% in most cases.
// compared to the real log10 function for 2<x<100.
// input: x=2<x<100 
// output: 100*log10(x) 
// Idea by Guido Socher
int log10_approx(unsigned char x)
{
	int l,log;
	if (x==1){
		return(0);
	}
	if (x<8){
		return(11*x+11);
	}
	//
	log=980-980/x;
	log/=10;
	if (x<31){
		l=19*x;
		l=l/10;
		log+=l-4;
	}else{
		l=67*x;
		l=l/100;
		if (x>51 && x<81){
			log+=l +42;
		}else{
			log+=l +39;
		}
	}
	if (log>200) log=200;
	return(log);
}
// and here calculates dew point
// input: humidity [in %RH], temperature [in C times 10]
// output: dew point [in C times 10]
int calc_dewpoint(int rh,int t)
{
		// we use integer math and everything times 100
        int k,tmp;
        k = ((100*log10_approx(rh)-20000))/43;
		// we do the calculations in many steps otherwise the compiler will try
		// to optimize and that creates nonsence as the numbers
		// can get too big or too small.
	//	usart_write("rH: %i t: %i 10_log(rH): %i k: %i \r\n",rh,t,log10_approx(rh),k);
        tmp=t/10;
        tmp=881*tmp;
        tmp=tmp/(243+t/10);
        k+=tmp*2;
        tmp=(1762-k)/10;
        tmp=24312/tmp;
        tmp=(tmp*k)/10;
        // dew point temp rounded:
		if (tmp<0){
		tmp-=51;
		}else{
		tmp+=51;
		}
        return (tmp/10);
}

/*
//--------------------------------------------------------------------
void sht_switch_heating_element(unsigned char onoff, unsigned char pin)
//--------------------------------------------------------------------
// shwitches the internal heating element
// input onoff 0:off 1:on 
{
	unsigned char status;
	unsigned char checksum;
	sht_read_statusreg(&status, &checksum, pin);
	if (onoff==0) status &= ~(HEATER_BIT);
	else status |= (HEATER_BIT);
	sht_write_statusreg(&status, pin);
}
*/

