// --- [ rs232.c ] ------------------------------------------------------------
//

#include <avr/io.h>
#include <avr/pgmspace.h>  
#include "rs232.h"
#include "..\global.h"

// ----------------------------------------------------------------------------
// initialisiert die RS232-Schnittstelle (ATMEGAxx8
void rs232_init(void)
{

UBRR0L = TEILER;									// siehe rs232_int.h
UBRR0H = 0;
UCSR0C = (0<<UMSEL01) | (1<<UCSZ01) | (1<<UCSZ00);

DISABLE_RX();
}


// --- Standard-Routinen ------------------------------------------------------
// ----------------------------------------------------------------------------
// ein einzelnes Zeichen auf COM ausgeben
void put_c(uint8_t Zeichen)
{
while(!(UCSR0A & (1<<UDRE0)));
UDR0 = Zeichen;
}

// ----------------------------------------------------------------------------
// einen String auf COM ausgeben
void put_s(char *s)
{
while(*s)
	{
	put_c(*s);
	s++;
	}
}

/*
// ----------------------------------------------------------------------------
// gibt einen String aus dem Flash aus
void puts_p(const char *progmem_s)
{
register char c;
while ( (c = pgm_read_byte(progmem_s++)) )
	{
   put_c(c);
	}
}
*/

/******************************************************************************
   Function    : uart_init()
   Parameters  :
   Return      :
   Description : init USART  with 8N1 / and UART_BAUDRATE

******************************************************************************/
void uart_init(void)
{
    /* Set the baud rate */
#if F_CPU < 2000000UL && defined(U2X)
    UCSR0A = _BV(U2X);             /* improve baud rate error by using 2x clk */
    UBRR0L = (F_CPU / (8UL * UART_BAUDRATE)) - 1;
#else
    UBRR0L = (F_CPU / (16UL * UART_BAUDRATE)) - 1;
#endif

    /* Enable UART receiver and transmitter */
    UCSR0B = (1 << RXEN0) | (1 << TXEN0);

    /* set to 8 data bits, 1 stop bit */
#if OUT_ELV
    UCSR0C = (1 << UCSZ01) | (1 << UCSZ00) | (1 << USBS0) | (1 << UPM01) | (1 << UPM00); //8;2;odd
#else
    UCSR0C = (1 << UCSZ01) | (1 << UCSZ00);	// original 8,1,no
#endif
}

void uart_putchar(char c)
{

    if (c == '\n')
    {
        uart_putchar('\r');
    }

    //loop_until_bit_is_set(UCSRA, UDRE);
    /* Wait for empty transmit buffer */
    while (!(UCSR0A & (1 << UDRE0)));

    /* transmit */
    UDR0 = c;
}

void uart_putbyte(char c)
{
    //loop_until_bit_is_set(UCSRA, UDRE);
    /* Wait for empty transmit buffer */
    while (!(UCSR0A & (1 << UDRE0)));

    /* transmit */
    UDR0 = c;
}

/* put 4bits as a hex quartet / ASCII */
void uart_puthex(uint8_t q)
{
    /* hexa */
    if (q < 10)
    {
        uart_putchar('0' + q);
    }
    else
    {
        uart_putchar('a' + (q - 10));
    }
}

/*
// output 3digts from integer with trailing ' ' (blank char) / ASCII 
void uart_putint(uint16_t q)
{
		if ( q >=1000 ) {			// gusts may become up to 183.6 kmh
		uart_puthex(q/1000);
		q = q%1000;
		}
		uart_puthex(q/100);
		uart_puthex((q/10)%10);
		uart_puthex(q%10);
		uart_putchar(' ');
}
*/

#if OUT_ASCII
// put ASCII-string 
void uart_putstr(const char* str)
{
    char* s = (char *)str;

    while (*s)
    {
        uart_putchar(*s);
        s++;
    }
}
#endif		//OUT_ASCII

// --- [ eof ] ----------------------------------------------------------------

